#ifndef _DISSECT_H_
#define _DISSECT_H_

//inclusions////////////////////////////////////////////////////////////////////
#include "vga.h"
#include "ethernet.h"
#include "log.h"

//variables/////////////////////////////////////////////////////////////////////
extern unsigned short int verbosity;
extern unsigned short int hours, minutes, seconds;

//structure to hold a dissector/////////////////////////////////////////////////
//+The typeid of the dissector is the number used by the lower-level protocol to
//identify it.  In the case of IP, this is the ethernet frame type # 0x800
//For SMTP, it would be the TCP destination port 25, etc.
//+Name is a null-terminated string with the protocol name in parentheses
//+dissectFcn is the function used to parse the associated protocol.
struct dissector
{
  long unsigned int typeid;
  unsigned char name[20];
  void (*dissectFcn)(unsigned char* data, unsigned int len); 
}dissector;

//function prototypes///////////////////////////////////////////////////////////
void parse_eth_frame(unsigned char* data, unsigned int len); //process received ethernet frames
void parse_ip_pkt(unsigned char* data, unsigned int len);    //process received IP packets
void parse_arp(unsigned char* data, unsigned int len);       //process received ARP messages
void parse_tcp_dgram(unsigned char* data, unsigned int len); //process received TCP datagrams
void parse_udp_dgram(unsigned char* data, unsigned int len); //process received UDP datagrams
void parse_icmp(unsigned char* data, unsigned int len);      //process received ICMP messages
void parse_ftp_data(unsigned char* data, unsigned int len);
void parse_ftp_control(unsigned char* data, unsigned int len);
void parse_telnet(unsigned char* data, unsigned int len);
void parse_smtp(unsigned char* data, unsigned int len);
void parse_dns(unsigned char* data, unsigned int len);
void parse_bootp_dhcp_server(unsigned char* data, unsigned int len);
void parse_bootp_dhcp_client(unsigned char* data, unsigned int len);
void parse_tftp(unsigned char* data, unsigned int len);
void parse_http(unsigned char* data, unsigned int len);
void parse_pop3(unsigned char* data, unsigned int len);
void parse_ntp(unsigned char* data, unsigned int len);
void parse_imap(unsigned char* data, unsigned int len);
void parse_aim(unsigned char* data, unsigned int len);

#endif //_DISSECT_H_
