#ifndef KEYBOARD_C
#define KEYBOARD_C

#include "keyboard.h"

//command LUT///////////////////////////////////////////////////////////////////
struct command cmd_table[] = 
{
  //commands that affect capture, filtering, logging or verbosity
  {"capture ",setCapture},
  {"filter ",setFilter},
  {"log ",setLog},
  {"verbosity ", setVerbosity},
  //commands that display stuff and take no arguments
  {"sysinfo",sysinfo},
  {"splash",splash},
  {"praise",praise},
  {"help",help},
  {"homer",homer},
  //commands that set the appearance of the status bar
  {"status ", setStatus},
  //commands to load/save user config
  {"config", config_cmd},
  //error if no command matched
  {"",cmdErrMsg}
};

//init the keyboard (just clear it)/////////////////////////////////////////////
void kybd_init()
{
  IOWR(KEYBOARD_BASE,0,KYBD_CLEAR);
  IOWR(KEYBOARD_BASE,0,0);
}

//compare two null-terminated strings///////////////////////////////////////////
//this checks if the first characters of the longer string are the same as the shorter string
int strcmpbeg(unsigned char * a, unsigned char * b)
{
  while((*a !='\0') && (*b !='\0'))
  {
    if(*a != *b)
    {
      return 0; 
    }
    a++;
    b++; 
  }
  return 1;
}

//read in the keyboard input, print & process the command///////////////////////
void kybd_read()
{
  alt_irq_disable(PROCESS_TIMER_IRQ);
  //read in the user input
  unsigned int i;
  unsigned char input[70];
  for(i=0;i<70;i++) 
  {
    IOWR(KEYBOARD_BASE,0,i+10);
    input[i]=IORD(KEYBOARD_BASE,0);
  }
  
  //null terminate it
  input[i]='\0';
  
  //clear the keyboard
  IOWR(KEYBOARD_BASE,0,KYBD_CLEAR);
  IOWR(KEYBOARD_BASE,0,0);
  
  //clear the interrupt
  IOWR(KEYBOARD_BASE,3,0);
  IOWR(KEYBOARD_BASE,2,0x100);
  
  //if we are showing the log, quit stops showing the log and anything
  //else shows the next page of log data
  if(showingLog==1)
  {
    if(strcmpbeg(input,"quit"))
    {
      showingLog=0; 
      readOffset=19;
    }
    else
    {
      log_show();
    }
  }
  else
  {
    //go through the command table and figure out what to do
    i=0;
    while(strcmpbeg(input,cmd_table[i].command)==0)
    {
      i++;
    }
    cmd_table[i].cmdFcn(input);
  }
  
  //reenable the process timer irq
  alt_irq_enable(PROCESS_TIMER_IRQ);
}

#endif
