#ifndef VGA_C
#define VGA_C

#include "vga.h"

//initialize vga////////////////////////////////////////////////////////////////
inline void vga_init(void)
{
   int i;
   for(i=0;i<64;i++)
   {
      vga_output("\n");
   }
   vga_status("\n");
}

//write char to vga output//////////////////////////////////////////////////////
inline void vga_write_output(char a)
{
  IOWR(VGA_BASE,0,a|VGA_WRITE);   //write one char to vga
  asm("nop;");                    //wait 1 clock cycle
  asm("nop;");
  IOWR(VGA_BASE,0,0x000);         //clear write bit
  asm("nop;");
  asm("nop;");                    //wait 1 clock cycle
}

//write one hex byte to VGA/////////////////////////////////////////////////////
inline void vga_hex_output(int i, int prefix)
{
  if(prefix)
  {
    vga_write_output('0');
    vga_write_output('x');
  }
  if((i&0x0F0) < 0x0A0)
  {
    vga_write_output(((0x0F0&i)/16)|0x030); 
  } 
  else
  {
    vga_write_output((((0x0F0&i)/16)-9)|0x040);
  }
  if((i&0x00F) < 0x00A)
  {
    vga_write_output((0x00F&i)|0x030); 
  } 
  else
  {
    vga_write_output(((0x00F&i)-9)|0x040);
  }
}

//print decimal numbers up to 99999 to VGA output///////////////////////////////
void vga_dec_output(int i)
{
   int trail=0, j=0;
   if((j=i/10000) || trail)
   {
      vga_write_output((j)|0x30);
      trail = 1;
      i=i%10000;
   }
   if((j=i/1000) || trail)
   {
      vga_write_output((j)|0x30);
      trail = 1;
      i=i%1000;
   }
   if((j=i/100) || trail)
   {
      vga_write_output((j)|0x30);
      trail = 1;
      i=i%100;
   }
   if((j=i/10) || trail)
   {
      vga_write_output((j)|0x30);
      trail = 1;
      i=i%10;
   }
   vga_write_output(i|0x30);
}

//write char to vga status//////////////////////////////////////////////////////
inline void vga_write_status(char a)
{
  IOWR(VGA_BASE,0,a|VGA_WRITE|VGA_STATUS);   //write one char to vga
  asm("nop;");
  asm("nop;");
  IOWR(VGA_BASE,0,0x000);     //clear write bit
  asm("nop;");
  asm("nop;");
}

//line return to vga output/////////////////////////////////////////////////////
inline void vga_return_output(void)
{
  IOWR(VGA_BASE,0,VGA_ENTER);
  asm("nop;");
  asm("nop;");
  IOWR(VGA_BASE,0,0x000);
  asm("nop;");
  asm("nop;");
  int i;
  for(i=0;i<128;i++)
  {
    vga_write_output(0x20); 
  }
}

//line return to vga status/////////////////////////////////////////////////////
inline void vga_return_status(void)
{
  IOWR(VGA_BASE,0,VGA_ENTER|VGA_STATUS);
  asm("nop;");
  asm("nop;");
  IOWR(VGA_BASE,0,0x000);
  asm("nop;");
  asm("nop;");
  int i;
  for(i=0;i<77;i++)
  {
    vga_write_status(0x20); 
  }
  IOWR(VGA_BASE,0,VGA_ENTER|VGA_STATUS);
  asm("nop;");
  asm("nop;");
  IOWR(VGA_BASE,0,0x000);
  asm("nop;");
  asm("nop;");
  
}

//print string to vga output////////////////////////////////////////////////////
void vga_output(const char* ptr)
{
  while(*ptr != '\0')
  {
    if(*ptr == '\n')
    {
      vga_return_output();
    }
    else
    {
      vga_write_output(*ptr);
    }
    ptr++;
  } 
}

//print string to vga status////////////////////////////////////////////////////
void vga_status(const char* ptr)
{
  while(*ptr != '\0')
  {
    if(*ptr == '\n')
    {
      vga_return_status();
    }
    else
    {
      vga_write_status(*ptr);
    }
    ptr++;
  } 
}

//write one hex byte to VGA status//////////////////////////////////////////////
inline void vga_hex_status(int i, int prefix)
{
  if(prefix)
  {
    vga_write_status('0');
    vga_write_status('x');
  }
  if((i&0x0F0) < 0x0A0)
  {
    vga_write_status(((0x0F0&i)/16)|0x030); 
  } 
  else
  {
    vga_write_status((((0x0F0&i)/16)-9)|0x040);
  }
  if((i&0x00F) < 0x00A)
  {
    vga_write_status((0x00F&i)|0x030); 
  } 
  else
  {
    vga_write_status(((0x00F&i)-9)|0x040);
  }
}

//print decimal numbers up to 99999 to VGA status///////////////////////////////
void vga_dec_status(int i, int zero_lead)
{
   int trail=0, j=0;
   if((j=i/10000) || zero_lead > 3 || trail)
   {
      vga_write_status((j)|0x30);
      trail = 1;
      i=i%10000;
   }
   if((j=i/1000) || zero_lead > 2|| trail)
   {
      vga_write_status((j)|0x30);
      trail = 1;
      i=i%1000;
   }
   if((j=i/100) || zero_lead > 1 || trail)
   {
      vga_write_status((j)|0x30);
      trail = 1;
      i=i%100;
   }
   if((j=i/10) || zero_lead > 0 || trail)
   {
      vga_write_status((j)|0x30);
      trail = 1;
      i=i%10;
   }
   vga_write_status(i|0x30);
}


#endif
